# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

import logging
import sys
from .error_holder import ErrorHolder

logger = logging.getLogger(__name__)


class _TestSuiteState(object):

    def __init__(self, result):
        self._result = result
        self._previous_class = None
        self._module_setup_failed = False
        self._class_setup_failed = False

    def _run_setup(self, item, setup_name, error_name):
        setup = getattr(item, setup_name, lambda: None)
        try:
            setup()
        except Exception:
            error = '{0} ({1})'.format(setup_name, error_name)
            self._result.addError(ErrorHolder(error), sys.exc_info())
            return False
        return True

    def _setup_module(self, module_name):
        if self._previous_class is not None:
            previous_module = self._previous_class.__module__
            if previous_module == module_name:
                return
            self._teardown_module(previous_module)

        module = sys.modules.get(module_name)
        if module is None:
            return

        logger.debug('Set up module: %r', module_name)
        self._module_setup_failed = not self._run_setup(
            module, 'setUpModule', module_name)

    def _setup_class(self, current_class):
        previous_class = self._previous_class
        if previous_class == current_class:
            logger.debug('Class has not changed; not setting up class %r',
                         current_class)
            return
        if self._module_setup_failed:
            logger.debug('Module setup failed; not setting up class %r',
                         previous_class)
            return
        if getattr(current_class, '__unittest_skip__', False):
            logger.debug('Class skipped; not setting up class %r',
                         current_class)
            return

        logger.debug('Set up class: %r', current_class)
        self._class_setup_failed = not self._run_setup(
            current_class, 'setUpClass', current_class.__name__)

    def setup(self, test):
        if isinstance(test, TestSuite):
            return True
        logger.debug('Setup module and class for %r', test)
        current_class = test.__class__
        module = current_class.__module__
        self._teardown_previous_class(current_class)
        self._setup_module(module)
        self._setup_class(current_class)

        self._previous_class = current_class

        return not (self._class_setup_failed or self._module_setup_failed)

    def _teardown_previous_class(self, current_class):
        previous_class = self._previous_class
        if previous_class is None:
            logger.debug('No previous class to tear down')
            return
        if current_class == self._previous_class:
            logger.debug('Class has not changed; not tearing down class %r',
                         previous_class)
            return
        if self._class_setup_failed:
            logger.debug(
                'Previous class setup failed; not tearing down class %r',
                previous_class)
            self._class_setup_failed = False
            return
        if self._module_setup_failed:
            logger.debug('Module setup failed; not tearing down class %r',
                         previous_class)
            return
        if getattr(previous_class, '__unittest_skip__', False):
            logger.debug('Previous class skipped; not tearing down class %r',
                         previous_class)
            return

        logger.debug('Tear down previous class: %r', previous_class)
        self._run_setup(
            previous_class, 'tearDownClass', previous_class.__name__)

    def _teardown_module(self, module_name):
        if self._module_setup_failed:
            logger.debug('Module setup failed; not tearing down module %r',
                         module_name)
            self._module_setup_failed = False
            return

        module = sys.modules.get(module_name)
        if module is None:
            return

        self._run_setup(module, 'tearDownModule', module_name)

    def teardown(self):
        if self._previous_class is None:
            return
        self._teardown_previous_class(None)
        previous_module = self._previous_class.__module__
        self._teardown_module(previous_module)


class TestSuite(object):
    """A ``TestSuite`` is a container of test cases and allows executing
    many test cases while managing the state of the overall suite.

    """

    def __init__(self, tests=()):
        self._tests = tuple(tests)

    def __iter__(self):
        return iter(self._tests)

    def __eq__(self, other):
        if not isinstance(other, TestSuite):
            return NotImplemented
        return list(self) == list(other)

    def __ne__(self, other):
        return not (self == other)

    def __call__(self, *args, **kwds):
        """Run all tests in the suite.

        Parameters
        ----------
        result : unittest.result.TestResult

        """
        return self.run(*args, **kwds)

    def run(self, result, _state=None):
        """Run all tests in the suite.

        Parameters
        ----------
        result : unittest.result.TestResult

        """
        if _state is None:
            state = _TestSuiteState(result)
        else:
            state = _state
        kwargs = {}
        for test in self:
            if state.setup(test):
                if isinstance(test, TestSuite):
                    kwargs = {'_state': state}
                test(result, **kwargs)
        if _state is None:
            state.teardown()
        return result

    def countTestCases(self):
        """Return the total number of tests contained in this suite.

        """
        return sum(test.countTestCases() for test in self)



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
import sys  # pragma: no cover

from .main import main  # pragma: no cover


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals  # pragma: no cover

import sys  # pragma: no cover

from .haas_application import HaasApplication  # pragma: no cover


def main():  # pragma: no cover
    """Execute haas.

    Parameters
    ----------
    argv : list
        The script's full argument list including the script itself.

    """
    application = HaasApplication(sys.argv)
    return application.run()



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

try:
    from haas.version import version as __version__
except ImportError:  # pragma: no cover
    __version__ = 'notset'

import logging


class NullHandler(logging.Handler):  # pragma: no cover
    def emit(self, record):
        pass


logger = logging.getLogger(__name__)
logger.addHandler(NullHandler())



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
__all__ = [
    'expected_failure',
    'unittest',
]

from contextlib import contextmanager
import sys


if sys.version_info[:2] == (2, 6):  # pragma: no cover
    import unittest2 as unittest
    from unittest2.case import _ExpectedFailure, _UnexpectedSuccess
else:  # pragma: no cover
    import unittest
    from unittest.case import _ExpectedFailure, _UnexpectedSuccess


@contextmanager
def expected_failure(condition=True):
    """Conditional expected failure context manager.

    Paramaters
    ----------
    condition : boolean
        If True, the context will be expected failure, otherwise it is
        expected to succeed.

    """
    if condition:
        try:
            yield
        except Exception:
            raise _ExpectedFailure(sys.exc_info())
        raise _UnexpectedSuccess()
    else:
        yield



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals


class PluginContext(object):
    """Handles correct setup and teardown of multiple plugins.

    """

    def __init__(self, hooks=None, **kwargs):
        super(PluginContext, self).__init__(**kwargs)
        if hooks is None:
            hooks = []
        self.hooks = tuple(filter(None, hooks))

    def __enter__(self):
        self.setup()

    def __exit__(self, exc_type, exc_value, traceback):
        self.teardown()

    def setup(self):
        for hook in self.hooks:
            hook.setup()

    def teardown(self):
        for hook in reversed(self.hooks):
            hook.teardown()



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import unicode_literals

import os

from setuptools import setup


VERSION = '0.1.0'


def write_version_py(filename=None):
    if filename is None:
        filename = os.path.join(os.path.dirname(__file__),
                                'haas', 'version.py')
    ver = """\
version = '{version}'
"""
    fh = open(filename, 'wb')
    try:
        fh.write(ver.format(version=VERSION).encode('utf-8'))
    finally:
        fh.close()


write_version_py()


with open('README.rst') as fh:
    long_description = fh.read()


setup(
    name='haas',
    version=VERSION,
    url='https://github.com/sjagoe/haas',
    author='Simon Jagoe',
    author_email='simon@simonjagoe.com',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: Testing',
    ],
    description='Extensible Python Test Runner',
    long_description=long_description,
    license='BSD',
    packages=['haas'],
    entry_points={
        'console_scripts': [
            'haas=haas.main:main',
        ],
    },
)



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2014 Simon Jagoe
# All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE.txt file for details.
from __future__ import absolute_import, unicode_literals

import haas
import logging
import sys


LEVELS = {
    'debug': logging.DEBUG,
    'info': logging.INFO,
    'warning': logging.WARNING,
    'error': logging.ERROR,
    'fatal': logging.FATAL,
    'critical': logging.CRITICAL,
}


def configure_logging(level):
    actual_level = LEVELS.get(level, logging.WARNING)
    format_ = '%(asctime)s %(levelname)-8.8s [%(name)s:%(lineno)s] %(message)s'
    formatter = logging.Formatter(format_)
    handler = logging.StreamHandler()
    handler.setFormatter(formatter)
    handler.setLevel(actual_level)
    logger = logging.getLogger(haas.__name__)
    logger.addHandler(handler)
    logger.setLevel(actual_level)
    logger.info('Logging configured for haas at level %r',
                logging.getLevelName(actual_level))


def get_module_by_name(name):
    """Import a module and return the imported module object.

    """
    __import__(name)
    return sys.modules[name]
